@@ -19,30 +19,26 @@
 
 ;;; Commentary:
 ;; Keypad state is a special state to simulate C-x and C-c key sequences.
-;; There are three commands:
+;;
+;; Useful commands:
+;;
+;; meow-keypad
+;; Enter keypad state.
 ;;
 ;; meow-keypad-start
 ;; Enter keypad state, and simulate this key with Control modifier.
 ;;
-;; meow-keypad-self-insert
-;; This command is bound to every single key in keypad state.
-;; The rules,
-;; - If current key is SPC, the next will be considered without modifier.
-;; - If current key is m, the next will be considered with Meta modifier.
-;; - Other keys, or SPC and m after a prefix, means append a key input, by default, with Control modifier.
-;;
-;; meow-keypad-undo
-;; Remove the last input, if there's no input in the sequence, exit the keypad state.
-
 ;;; Code:
 
+(require 'cl-lib)
 (require 'subr-x)
 (require 'meow-var)
 (require 'meow-util)
 (require 'meow-helpers)
+(require 'meow-beacon)
 
 (defun meow--keypad-format-upcase (k)
-  "Return S-k for upcase k."
+  "Return S-k for upcase K."
   (let ((case-fold-search nil))
     (if (and (stringp k)
              (string-match-p "^[A-Z]$" k))
@@ -67,13 +63,16 @@
    (t "")))
 
 (defun meow--keypad-lookup-key (keys)
-  (let* ((overriding-local-map meow--keypad-base-keymap)
-         (keybind (key-binding keys)))
-    (unless (and (meow--is-self-insertp keybind)
-                 (not meow-keypad-self-insert-undefined))
-      keybind)))
+  "Lookup the command which is bound at KEYS."
+  (let* ((keybind (if meow--keypad-base-keymap
+		      (lookup-key meow--keypad-base-keymap keys)
+		    (key-binding keys))))
+    keybind))
 
 (defun meow--keypad-has-sub-meta-keymap-p ()
+  "Check if there's a keymap belongs to Meta prefix.
+
+A key sequences starts with ESC is accessible via Meta key."
   (and (not meow--use-literal)
        (not meow--use-both)
        (not meow--use-meta)
@@ -84,7 +83,9 @@
                   (lookup-key keymap ""))))))
 
 (defun meow--keypad-format-keys (&optional prompt)
-  "Return a display format for current input keys."
+  "Return a display format for current input keys.
+
+The message is prepended with an optional PROMPT."
   (let ((result ""))
     (setq result
           (thread-first
@@ -116,8 +117,10 @@
         meow--use-meta nil
         meow--use-both nil
         meow--keypad-help nil)
-  (setq overriding-local-map nil)
-  (meow--exit-keypad-state))
+  (meow--keypad-clear-message)
+  (meow--exit-keypad-state)
+  ;; Return t to indicate the keypad loop should be stopped
+  t)
 
 (defun meow-keypad-quit ()
   "Quit keypad state."
@@ -128,6 +131,9 @@
   (meow--keypad-quit))
 
 (defun meow--make-keymap-for-describe (keymap control)
+  "Parse the KEYMAP to make it suitable for describe.
+
+Argument CONTROL, non-nils stands for current input is prefixed with Control."
   (let ((km (make-keymap)))
     (suppress-keymap km t)
     (when (keymapp keymap)
@@ -142,6 +148,7 @@
     km))
 
 (defun meow--keypad-get-keymap-for-describe ()
+  "Get a keymap for describe."
   (let* ((input (thread-first
                   (mapcar #'meow--keypad-format-key-1 meow--keypad-keys)
                   (reverse)
@@ -159,7 +166,7 @@
       (when meta-both-keymap
         (meow--make-keymap-for-describe meta-both-keymap t)))
      (meow--use-literal
-      (when-let ((keymap (meow--keypad-lookup-key (read-kbd-macro input))))
+      (when-let* ((keymap (meow--keypad-lookup-key (read-kbd-macro input))))
         (when (keymapp keymap)
           (meow--make-keymap-for-describe keymap nil))))
 
@@ -171,10 +178,10 @@
      ;; Leader keymap may contain meow-dispatch commands
      ;; translated names based on the commands they refer to
      ((null meow--keypad-keys)
-      (when-let ((keymap (if (stringp meow-keypad-leader-dispatch)
-                             (meow--keypad-lookup-key (read-kbd-macro meow-keypad-leader-dispatch))
-                           (or meow-keypad-leader-dispatch
-                               (alist-get 'leader meow-keymap-alist)))))
+      (when-let* ((keymap (if (stringp meow-keypad-leader-dispatch)
+                              (meow--keypad-lookup-key (read-kbd-macro meow-keypad-leader-dispatch))
+                            (or meow-keypad-leader-dispatch
+                                (alist-get 'leader meow-keymap-alist)))))
         (let ((km (make-keymap)))
           (suppress-keymap km t)
           (map-keymap
@@ -191,7 +198,7 @@
           km)))
 
      (t
-      (when-let ((keymap (meow--keypad-lookup-key (read-kbd-macro input))))
+      (when-let* ((keymap (meow--keypad-lookup-key (read-kbd-macro input))))
         (when (keymapp keymap)
           (let* ((km (make-keymap))
                  (has-sub-meta (meow--keypad-has-sub-meta-keymap-p))
@@ -221,16 +228,21 @@
              keymap)
             km)))))))
 
-(defun meow--keypad-display-message ()
-  (let (overriding-local-map)
-    (when meow-keypad-describe-keymap-function
-      (when (or
-             meow--keypad-keymap-description-activated
+(defun meow--keypad-clear-message ()
+  "Clear displayed message by calling `meow-keypad-clear-describe-keymap-function'."
+  (when meow-keypad-clear-describe-keymap-function
+    (funcall meow-keypad-clear-describe-keymap-function)))
 
-             (setq meow--keypad-keymap-description-activated
-                   (sit-for meow-keypad-describe-delay t)))
-        (let ((keymap (meow--keypad-get-keymap-for-describe)))
-          (funcall meow-keypad-describe-keymap-function keymap))))))
+(defun meow--keypad-display-message ()
+  "Display a message for current input state."
+  (when meow-keypad-describe-keymap-function
+    (when (or
+           meow--keypad-keymap-description-activated
+
+           (setq meow--keypad-keymap-description-activated
+                 (sit-for meow-keypad-describe-delay t)))
+      (let ((keymap (meow--keypad-get-keymap-for-describe)))
+        (funcall meow-keypad-describe-keymap-function keymap)))))
 
 (defun meow--describe-keymap-format (pairs &optional width)
   (let* ((fw (or width (frame-width)))
@@ -317,8 +329,9 @@
               (max-mini-window-height 1.0))
           (save-window-excursion
             (with-temp-message
-                (format "%s\nKEYPAD: %s%s"
+                (format "%s\n%s%s%s"
                         msg
+                        meow-keypad-message-prefix
                         (let ((pre (meow--keypad-format-prefix)))
                           (if (string-blank-p pre)
                               ""
@@ -330,9 +343,9 @@
   "Return a symbol as title or DEF.
 
 Returning DEF will result in a generated title."
-  (if-let ((cmd (and (symbolp def)
-                     (commandp def)
-                     (get def 'meow-dispatch))))
+  (if-let* ((cmd (and (symbolp def)
+                      (commandp def)
+                      (get def 'meow-dispatch))))
       (meow--keypad-lookup-key (kbd cmd))
     def))
 
@@ -358,55 +371,111 @@ Returning DEF will result in a generated title."
     (meow--keypad-quit)))
 
 (defun meow--keypad-show-message ()
+  "Show message for current keypad input."
   (let ((message-log-max))
-    (message "KEYPAD%s: %s%s"
-             (if meow--keypad-help " describe key" "")
+    (message "%s%s%s%s"
+             meow-keypad-message-prefix
+             (if meow--keypad-help "(describe key)" "")
              (let ((pre (meow--keypad-format-prefix)))
                (if (string-blank-p pre)
                    ""
                  (propertize pre 'face 'font-lock-comment-face)))
              (propertize (meow--keypad-format-keys nil) 'face 'font-lock-string-face))))
 
-(defun meow--keypad-try-execute ()
-  "Try execute command.
+(defun meow--keypad-in-beacon-p ()
+  "Return whether keypad is started from BEACON state."
+  (and (meow--beacon-inside-secondary-selection)
+       meow--beacon-overlays))
 
-If there is a command available on the current key binding,
-try replacing the last modifier and try again."
+(defun meow--keypad-execute (command)
+  "Execute the COMMAND.
+
+If there are beacons, execute it at every beacon."
+  (if (meow--keypad-in-beacon-p)
+      (cond
+       ((member command '(kmacro-start-macro kmacro-start-macro-or-insert-counter))
+        (call-interactively 'meow-beacon-start))
+       ((member command '(kmacro-end-macro meow-end-kmacro))
+        (call-interactively 'meow-beacon-end-and-apply-kmacro))
+       ((and (not defining-kbd-macro)
+             (not executing-kbd-macro)
+             meow-keypad-execute-on-beacons)
+        (call-interactively command)
+        (meow--beacon-apply-command command)))
+    (call-interactively command)))
+
+(defun meow--keypad-try-execute ()
+  "Try execute command, return t when the translation progress can be ended.
+
+This function supports a fallback behavior, where it allows to use `SPC
+x f' to execute `C-x C-f' or `C-x f' when `C-x C-f' is not bound."
   (unless (or meow--use-literal
               meow--use-meta
               meow--use-both)
     (let* ((key-str (meow--keypad-format-keys nil))
-           (cmd (meow--keypad-lookup-key (read-kbd-macro key-str))))
+           (cmd (meow--keypad-lookup-key (kbd key-str))))
       (cond
+       ((keymapp cmd)
+        (when meow-keypad-message (meow--keypad-show-message))
+        (meow--keypad-display-message)
+        nil)
        ((commandp cmd t)
         (setq current-prefix-arg meow--prefix-arg
               meow--prefix-arg nil)
         (if meow--keypad-help
             (progn
               (meow--keypad-quit)
-              (describe-function cmd))
+              (describe-function cmd)
+              t)
           (let ((meow--keypad-this-command cmd))
             (meow--keypad-quit)
             (setq real-this-command cmd
                   this-command cmd)
-            (call-interactively cmd))))
-       ((keymapp cmd)
-        (when meow-keypad-message (meow--keypad-show-message))
-        (meow--keypad-display-message))
+            (meow--keypad-execute cmd)
+            t)))
        ((equal 'control (caar meow--keypad-keys))
         (setcar meow--keypad-keys (cons 'literal (cdar meow--keypad-keys)))
         (meow--keypad-try-execute))
        (t
         (setq meow--prefix-arg nil)
-        (message "%s is undefined" (meow--keypad-format-keys nil))
-        (meow--keypad-quit))))))
+        (meow--keypad-quit)
+        (if (or (eq t meow-keypad-leader-transparent)
+                (eq meow--keypad-previous-state meow-keypad-leader-transparent))
+          (let* ((key (meow--parse-input-event last-input-event))
+                 (origin-cmd (cl-some (lambda (m)
+                                        (when (and (not (eq m meow-normal-state-keymap))
+                                                   (not (eq m meow-motion-state-keymap)))
+                                          (let ((cmd (lookup-key m (kbd key))))
+                                            (when (commandp cmd)
+                                              cmd))))
+                                      (current-active-maps)))
+                 (remapped-cmd (command-remapping origin-cmd))
+                 (cmd-to-call (if (member remapped-cmd '(undefined nil))
+                                  (or origin-cmd 'undefined)
+                                remapped-cmd)))
+            (meow--keypad-execute cmd-to-call))
+          (message "%s is undefined" key-str))
+        t)))))
 
-(defun meow-keypad-self-insert ()
-  "Default command when keypad state is enabled."
-  (interactive)
-  (setq this-command last-command)
-  (when-let ((e (meow--event-key last-input-event))
-             (key (meow--parse-input-event e)))
+(defun meow--keypad-handle-input-with-keymap (input-event)
+  "Handle INPUT-EVENT with `meow-keypad-state-keymap'.
+
+Return t if handling is completed."
+  (if (equal 'escape last-input-event)
+      (meow--keypad-quit)
+    (setq last-command-event last-input-event)
+    (let ((kbd (single-key-description input-event)))
+      (if-let* ((cmd (lookup-key meow-keypad-state-keymap (read-kbd-macro kbd))))
+          (call-interactively cmd)
+        (meow--keypad-handle-input-event input-event)))))
+
+(defun meow--keypad-handle-input-event (input-event)
+  "Handle the INPUT-EVENT.
+
+Add a parsed key and its modifier to current key sequence. Then invoke a
+command when there's one available on current key sequence."
+  (meow--keypad-clear-message)
+  (when-let* ((key (single-key-description input-event)))
     (let ((has-sub-meta (meow--keypad-has-sub-meta-keymap-p)))
       (cond
        (meow--use-literal
@@ -419,31 +488,29 @@ try replacing the last modifier and try again."
        (meow--use-meta
         (push (cons 'meta key) meow--keypad-keys)
         (setq meow--use-meta nil))
-       ((and (equal e meow-keypad-meta-prefix)
+       ((and (equal input-event meow-keypad-meta-prefix)
              (not meow--use-meta)
              has-sub-meta)
         (setq meow--use-meta t))
-       ((and (equal e meow-keypad-ctrl-meta-prefix)
+       ((and (equal input-event meow-keypad-ctrl-meta-prefix)
              (not meow--use-both)
              has-sub-meta)
         (setq meow--use-both t))
-       ((and (equal e meow-keypad-literal-prefix)
+       ((and (equal input-event meow-keypad-literal-prefix)
              (not meow--use-literal)
              meow--keypad-keys)
         (setq meow--use-literal t))
        (meow--keypad-keys
         (push (cons 'control key) meow--keypad-keys))
-       ((alist-get e meow-keypad-start-keys)
+       ((alist-get input-event meow-keypad-start-keys)
         (push (cons 'control (meow--parse-input-event
-                              (alist-get e meow-keypad-start-keys)))
+                              (alist-get input-event meow-keypad-start-keys)))
               meow--keypad-keys))
-       (meow--keypad-allow-quick-dispatch
-        (if-let ((keymap (meow--get-leader-keymap)))
+       (t
+        (if-let* ((keymap (meow--get-leader-keymap)))
             (setq meow--keypad-base-keymap keymap)
           (setq meow--keypad-keys (meow--parse-string-to-keypad-keys meow-keypad-leader-dispatch)))
-        (push (cons 'literal key) meow--keypad-keys))
-       (t
-        (push (cons 'control key) meow--keypad-keys))))
+        (push (cons 'literal key) meow--keypad-keys))))
 
     ;; Try execute if the input is valid.
     (if (or meow--use-literal
@@ -451,51 +518,53 @@ try replacing the last modifier and try again."
             meow--use-both)
         (progn
           (when meow-keypad-message (meow--keypad-show-message))
-          (meow--keypad-display-message))
+          (meow--keypad-display-message)
+          nil)
       (meow--keypad-try-execute))))
 
 (defun meow-keypad ()
-  "Enter keypad state."
+  "Enter keypad state and convert inputs."
   (interactive)
-  (setq this-command last-command)
-  (setq meow--keypad-previous-state (meow--current-state))
-  (meow--switch-state 'keypad)
-  (setq overriding-local-map meow-keypad-state-keymap
-        overriding-terminal-local-map nil)
-  (meow--keypad-display-message))
+  (meow-keypad-start-with nil))
 
 (defun meow-keypad-start ()
   "Enter keypad state with current input as initial key sequences."
   (interactive)
-  (setq this-command last-command)
-  (setq meow--keypad-previous-state (meow--current-state))
+  (setq this-command last-command
+        meow--keypad-keys nil
+        meow--keypad-previous-state (meow--current-state)
+        meow--prefix-arg current-prefix-arg)
   (meow--switch-state 'keypad)
-  (setq overriding-local-map meow-keypad-state-keymap
-        overriding-terminal-local-map nil
-        meow--keypad-allow-quick-dispatch nil)
-  (call-interactively 'meow-keypad-self-insert))
+  (unwind-protect
+      (progn
+        (meow--keypad-handle-input-with-keymap last-input-event)
+        (while (not (meow--keypad-handle-input-with-keymap (read-key)))))
+    (when (bound-and-true-p meow-keypad-mode)
+      (meow--keypad-quit))))
 
 (defun meow-keypad-start-with (input)
   "Enter keypad state with INPUT.
 
-INPUT is a string, stands for initial keys."
-  (setq meow--keypad-previous-state (meow--current-state))
+A string INPUT, stands for initial keys.
+When INPUT is nil, start without initial keys."
+  (setq this-command last-command
+        meow--keypad-keys (when input (meow--parse-string-to-keypad-keys input))
+        meow--keypad-previous-state (meow--current-state)
+        meow--prefix-arg current-prefix-arg)
   (meow--switch-state 'keypad)
-  (setq meow--keypad-keys (meow--parse-string-to-keypad-keys input)
-        overriding-terminal-local-map nil
-        overriding-local-map meow-keypad-state-keymap)
-  (meow--keypad-try-execute))
+  (unwind-protect
+      (progn
+        (meow--keypad-show-message)
+        (meow--keypad-display-message)
+        (while (not (meow--keypad-handle-input-with-keymap (read-key)))))
+    (when (bound-and-true-p meow-keypad-mode)
+      (meow--keypad-quit))))
 
 (defun meow-keypad-describe-key ()
   "Describe key via KEYPAD input."
   (interactive)
-  (setq this-command last-command)
-  (setq overriding-local-map meow-keypad-state-keymap
-        meow--keypad-help t
-        meow--keypad-previous-state (meow--current-state))
-  (meow--switch-state 'keypad)
-  (meow--keypad-show-message)
-  (meow--keypad-display-message))
+  (setq meow--keypad-help t)
+  (meow-keypad))
 
 (provide 'meow-keypad)
 ;;; meow-keypad.el ends here
