@@ -105,6 +105,26 @@ Argument ENABLE non-nil means turn on."
       (add-hook 'meow-insert-exit-hook #'meow--company-maybe-abort-advice)
     (remove-hook 'meow-insert-exit-hook #'meow--company-maybe-abort-advice)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; corfu
+
+(declare-function corfu-quit "corfu")
+
+(defvar meow--corfu-setup nil
+  "Whether already setup corfu.")
+
+(defun meow--corfu-maybe-abort-advice ()
+  "Adviced for `meow-insert-exit'."
+  (when (bound-and-true-p corfu-mode) (corfu-quit)))
+
+(defun meow--setup-corfu (enable)
+  "Setup for corfu.
+Argument ENABLE non-nil means turn on."
+  (setq meow--corfu-setup enable)
+  (if enable
+      (add-hook 'meow-insert-exit-hook #'meow--corfu-maybe-abort-advice)
+    (remove-hook 'meow-insert-exit-hook #'meow--corfu-maybe-abort-advice)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; repeat-map
 
@@ -147,6 +167,28 @@ Argument ENABLE non-nil means turn on."
     (advice-remove 'wgrep-finish-edit #'meow--switch-to-motion)
     (advice-remove 'wgrep-save-all-buffers #'meow--switch-to-motion)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; grep-edit
+
+
+(defvar meow--grep-edit-setup nil
+  "Wheter already setup grep-edit.")
+
+(defvar grep-edit-mode-hook)
+
+(declare-function grep-edit-save-changes "grep")
+
+(defun meow--setup-grep-edit (enable)
+  "Setup grep-edit.
+
+Argument ENABLE non-nil means turn on."
+  (if enable
+      (progn
+        (add-hook 'grep-edit-mode-hook #'meow--switch-to-normal)
+        (advice-add #'grep-edit-save-changes :after #'meow--switch-to-motion))
+    (remove-hook 'grep-edit-mode-hook #'meow--switch-to-normal)
+    (advice-remove 'grep-edit-save-changes #'meow--switch-to-motion)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; wdired
 
@@ -220,7 +262,7 @@ Argument ENABLE non-nil means turn on."
 (defvar meow--magit-setup nil)
 
 (defun meow--magit-blame-hook-function ()
-  "Switch meow state when entering/leaving magit-blame-read-only-mode."
+  "Switch meow state when entering/leaving `magit-blame-read-only-mode'."
   (if (bound-and-true-p magit-blame-read-only-mode)
       (meow--switch-to-motion)
     (meow--switch-to-normal)))
@@ -373,13 +415,22 @@ Argument ENABLE non-nil means turn on."
 (defvar meow--which-key-setup nil)
 
 (defun meow--which-key-describe-keymap ()
+  "Use which-key for keypad popup."
   (if which-key-mode
-      (setq meow-keypad-describe-keymap-function
+      (setq
+       which-key-use-C-h-commands nil
+       meow-keypad-describe-keymap-function
 	(lambda (keymap)
-	  (which-key--create-buffer-and-show nil keymap nil (concat "Meow: " (meow--keypad-format-keys)))))
-    (setq meow-keypad-describe-keymap-function 'meow-describe-keymap)))
+	  (which-key--create-buffer-and-show nil keymap nil (concat meow-keypad-message-prefix (meow--keypad-format-keys))))
+        meow-keypad-clear-describe-keymap-function 'which-key--hide-popup)
+
+    (setq meow-keypad-describe-keymap-function 'meow-describe-keymap
+          meow-keypad-clear-describe-keymap-function nil
+          which-key-use-C-h-commands t)))
 
 (defun meow--setup-which-key (enable)
+  "Setup which-key.
+Argument ENABLE non-nil means turn on."
   (setq meow--which-key-setup enable)
   (if enable
       (add-hook 'which-key-mode-hook 'meow--which-key-describe-keymap)
@@ -393,10 +444,14 @@ Argument ENABLE non-nil means turn on."
 (defun meow--input-method-advice (fnc key)
   "Advice for `quail-input-method'.
 
-Only use the input method in insert mode."
+Only use the input method in insert mode.
+Argument FNC, input method function.
+Argument KEY, the current input."
   (funcall (if (and (boundp 'meow-mode) meow-mode (not (meow-insert-mode-p))) #'list fnc) key))
 
 (defun meow--setup-input-method (enable)
+  "Setup input-method.
+Argument ENABLE non-nil means turn on."
   (setq meow--input-method-setup enable)
   (if enable
       (advice-add 'quail-input-method :around 'meow--input-method-advice)
@@ -449,6 +504,53 @@ Argument ENABLE non-nil means turn on."
       ;; These vars allow us the select through the polymode chunk
       (add-to-list 'polymode-move-these-vars-from-old-buffer v))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; eat-eshell
+
+(defvar meow--eat-eshell-setup nil)
+(defvar meow--eat-eshell-mode-override nil)
+
+(declare-function eat-eshell-emacs-mode "eat")
+(declare-function eat-eshell-semi-char-mode "eat")
+(declare-function eat-eshell-char-mode "eat")
+
+(declare-function meow-insert-mode "meow-core")
+
+(defun meow--eat-eshell-mode-override-enable ()
+  (setq-local meow--eat-eshell-mode-override t)
+  (add-hook 'meow-insert-enter-hook #'eat-eshell-char-mode nil t)
+  (add-hook 'meow-insert-exit-hook #'eat-eshell-emacs-mode nil t)
+  (if (bound-and-true-p meow-insert-mode)
+      (eat-eshell-char-mode)
+    (eat-eshell-emacs-mode)))
+
+(defun meow--eat-eshell-mode-override-disable ()
+  (setq-local meow--eat-eshell-mode-override nil)
+  (remove-hook 'meow-insert-enter-hook #'eat-eshell-char-mode t)
+  (remove-hook 'meow-insert-exit-hook #'eat-eshell-emacs-mode t))
+
+(defun meow--setup-eat-eshell (enable)
+  (setq meow--eat-eshell-setup enable)
+  (if enable
+      (progn (add-hook 'eat-eshell-exec-hook #'meow--eat-eshell-mode-override-enable)
+             (add-hook 'eat-eshell-exit-hook #'meow--eat-eshell-mode-override-disable)
+             (add-hook 'eat-eshell-exit-hook #'meow--update-cursor))
+
+    (remove-hook 'eat-eshell-exec-hook #'meow--eat-eshell-mode-override-enable)
+    (remove-hook 'eat-eshell-exit-hook #'meow--eat-eshell-mode-override-disable)
+    (remove-hook 'eat-eshell-exit-hook #'meow--update-cursor)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Ediff
+(defvar meow--ediff-setup nil)
+
+(defun meow--setup-ediff (enable)
+  "Setup Ediff.
+Argument ENABLE, non-nil means turn on."
+  (if enable
+      (add-hook 'ediff-mode-hook 'meow-motion-mode)
+    (remove-hook 'ediff-mode-hook 'meow-motion-mode)))
+
 ;; Enable / Disable shims
 
 (defun meow--enable-shims ()
@@ -465,7 +567,9 @@ Argument ENABLE non-nil means turn on."
   (eval-after-load "edebug" (lambda () (meow--setup-edebug t)))
   (eval-after-load "magit" (lambda () (meow--setup-magit t)))
   (eval-after-load "wgrep" (lambda () (meow--setup-wgrep t)))
+  (eval-after-load "grep" (lambda () (meow--setup-grep-edit t)))
   (eval-after-load "company" (lambda () (meow--setup-company t)))
+  (eval-after-load "corfu" (lambda () (meow--setup-corfu t)))
   (eval-after-load "polymode" (lambda () (meow--setup-polymode t)))
   (eval-after-load "cider" (lambda () (meow--setup-cider t)))
   (eval-after-load "sly" (lambda () (meow--setup-sly t)))
@@ -474,7 +578,9 @@ Argument ENABLE non-nil means turn on."
   (eval-after-load "undo-tree" (lambda () (meow--setup-undo-tree t)))
   (eval-after-load "diff-hl" (lambda () (meow--setup-diff-hl t)))
   (eval-after-load "quail" (lambda () (meow--setup-input-method t)))
-  (eval-after-load "skk" (lambda () (meow--setup-ddskk t))))
+  (eval-after-load "skk" (lambda () (meow--setup-ddskk t)))
+  (eval-after-load "eat" (lambda () (meow--setup-eat-eshell t)))
+  (eval-after-load "ediff" (lambda () (meow--setup-ediff t))))
 
 (defun meow--disable-shims ()
   "Remove shim setups."
@@ -486,13 +592,17 @@ Argument ENABLE non-nil means turn on."
   (when meow--edebug-setup (meow--setup-edebug nil))
   (when meow--magit-setup (meow--setup-magit nil))
   (when meow--company-setup (meow--setup-company nil))
+  (when meow--corfu-setup (meow--setup-corfu nil))
   (when meow--wgrep-setup (meow--setup-wgrep nil))
+  (when meow--grep-edit-setup (meow--setup-grep-edit nil))
   (when meow--polymode-setup (meow--setup-polymode nil))
   (when meow--cider-setup (meow--setup-cider nil))
   (when meow--which-key-setup (meow--setup-which-key nil))
   (when meow--diff-hl-setup (meow--setup-diff-hl nil))
   (when meow--input-method-setup (meow--setup-input-method nil))
-  (when meow--ddskk-setup (meow--setup-ddskk nil)))
+  (when meow--ddskk-setup (meow--setup-ddskk nil))
+  (when meow--eat-eshell-setup (meow--setup-eat-eshell nil))
+  (when meow--ediff-setup (meow--setup-ediff nil)))
 
 ;;; meow-shims.el ends here
 (provide 'meow-shims)
