@@ -51,18 +51,22 @@
 (declare-function meow--beacon-apply-command "meow-beacon")
 (declare-function meow-keypad-start-with "meow-keypad")
 
-(defun meow--execute-kbd-macro (kbd-macro)
-  "Execute KBD-MACRO."
-  (when-let ((ret (key-binding (read-kbd-macro kbd-macro))))
+(defun meow--execute-kbd-macro (kbd-macro-or-defun)
+  "Execute the function bound to `KBD-MACRO-OR-DEFUN'. If `KBD-MACRO-OR-DEFUN' is a string,
+instead execute the keyboard macro it corresponds to."
+  (when-let* ((ret (if (and (symbolp kbd-macro-or-defun) (fboundp kbd-macro-or-defun))
+                       kbd-macro-or-defun
+                     (key-binding (read-kbd-macro kbd-macro-or-defun)))))
     (cond
      ((commandp ret)
+      (setq this-command ret)
       (call-interactively ret))
 
      ((and (not meow-use-keypad-when-execute-kbd) (keymapp ret))
       (set-transient-map ret nil nil))
 
      ((and meow-use-keypad-when-execute-kbd (keymapp ret))
-      (meow-keypad-start-with kbd-macro)))))
+      (meow-keypad-start-with kbd-macro-or-defun)))))
 
 (defun meow-insert-mode-p ()
   "Whether insert mode is enabled."
@@ -342,7 +346,7 @@ Looks up the state in meow-replace-state-name-list"
   (member 'shift last-input-event))
 
 (defun meow--bounds-with-type (type thing)
-  (when-let ((bounds (bounds-of-thing-at-point thing)))
+  (when-let* ((bounds (bounds-of-thing-at-point thing)))
     (cons type bounds)))
 
 (defun meow--insert (&rest args)
@@ -431,7 +435,7 @@ Looks up the state in meow-replace-state-name-list"
   (setq-local meow-normal-mode nil)
   (when (or (member this-command meow-grab-fill-commands)
             (member meow--keypad-this-command meow-grab-fill-commands))
-    (when-let ((s (meow--second-sel-get-string)))
+    (when-let* ((s (meow--second-sel-get-string)))
       (meow--insert s))))
 
 (defun meow--parse-string-to-keypad-keys (str)
@@ -469,41 +473,6 @@ Looks up the state in meow-replace-state-name-list"
     (format "<%s>" e))
    (t nil)))
 
-(defun meow--save-origin-commands ()
-  "Save the commands overridden by the Motion map to modified bindings.
-
-The new key binding, modified by the prefix in
-`meow-motion-remap-prefix', is bound to a command that calls the
-command locally bound to the original key binding, or, if that is
-nil, the original command.
-
-For example, under the default and suggested settings, in a
-Magit status buffer, `k' could be bound to `meow-previous'
-and `H-k' would be bound to a command that would try
-to use the status buffer's original `k' binding at point."
-  (cl-loop for key-code being the key-codes of meow-motion-state-keymap do
-           (ignore-errors
-             (let* ((key (meow--parse-input-event key-code))
-                    (cmd (key-binding (kbd key))))
-               (when (and (commandp cmd)
-                          (not (equal cmd 'undefined)))
-                 (let ((rebind-key (concat meow-motion-remap-prefix key)))
-                   (local-set-key (kbd rebind-key)
-                                  (lambda ()
-                                    (interactive)
-                                    ;; Local maps are those local to the buffer
-                                    ;; or a region of the buffer.
-                                    (let* ((local (lookup-key (current-local-map) key))
-                                           (remapped (command-remapping local)))
-                                      (call-interactively
-                                       (cond
-                                        ((commandp remapped)
-                                         remapped)
-                                        ((commandp local)
-                                         local)
-                                        (t
-                                         cmd))))))))))))
-
 (defun meow--prepare-region-for-kill ()
   (when (and (equal 'line (cdr (meow--selection-type)))
              (meow--direction-forward-p)
@@ -526,6 +495,25 @@ to use the status buffer's original `k' binding at point."
         (upcase c)
       c)))
 
+
+
+(defun meow--make-button (string callback &optional data help-echo)
+  "Copy from buttonize, which is available in Emacs 29.1"
+  (let ((string
+         (apply #'propertize string
+                (list 'font-lock-face 'button
+                      'mouse-face 'highlight
+                      'help-echo help-echo
+                      'button t
+                      'follow-link t
+                      'category t
+                      'button-data data
+                      'keymap button-map
+                      'action callback))))
+    ;; Add the face to the end so that it can be overridden.
+    (add-face-text-property 0 (length string) 'button t string)
+    string))
+
 (defun meow--parse-def (def)
   "Return a command or keymap for DEF.
 
@@ -537,7 +525,8 @@ that bound to DEF. Otherwise, return DEF."
         (defalias cmd-name
           (lambda ()
             (:documentation
-             (format "Execute the command which is bound to %s." def))
+             (format "Execute the command which is bound to %s."
+                     (meow--make-button def 'describe-key (kbd def))))
             (interactive)
             (meow--execute-kbd-macro def)))
         (put cmd-name 'meow-dispatch def)
@@ -607,7 +596,7 @@ that bound to DEF. Otherwise, return DEF."
 
 (defun meow--remove-fake-cursor (rol)
   (when (overlayp rol)
-    (when-let ((ovs (overlay-get rol 'meow-face-cursor)))
+    (when-let* ((ovs (overlay-get rol 'meow-face-cursor)))
       (mapc (lambda (o) (when (overlayp o) (delete-overlay o)))
             ovs))))
 
